<?php
/**
 * WooCommerce Jilt
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@jilt.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce Jilt to newer
 * versions in the future. If you wish to customize WooCommerce Jilt for your
 * needs please refer to http://help.jilt.com/collection/176-jilt-for-woocommerce
 *
 * @package   WC-Jilt/Admin
 * @author    Jilt
 * @category  Admin
 * @copyright Copyright (c) 2015-2016, SkyVerge, Inc.
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

defined( 'ABSPATH' ) or exit;

/**
 * Main integration class
 *
 * @since 1.0.0
 */
class WC_Jilt_Integration extends WC_Integration {


	/** Debug mode log to file */
	const DEBUG_MODE_LOG = 'log';

	/** Debug mode disabled */
	const DEBUG_MODE_OFF = 'off';

	/** @var WC_Jilt_API instance */
	private $api;


	/**
	 * Initialize the class
	 *
	 * @since 1.0.0
	 * @see WC_Jilt_Integration::instance()
	 */
	public function __construct() {

		$this->id                 = 'jilt';
		$this->method_title       = __( 'Jilt', 'jilt-for-woocommerce' );
		$this->method_description = __( 'Automatically send reminder emails to customers who have abandoned their cart, and recover lost sales', 'jilt-for-woocommerce' );

		// Load admin form
		$this->init_form_fields();

		// Load settings
		$this->init_settings();

		add_filter( 'woocommerce_settings_api_sanitized_fields_jilt', array( $this, 'sanitize_fields' ) );

		if ( $this->is_linked() ) {

			// handle placed orders and keeping the financial status in sync with Jilt
			add_action( 'woocommerce_order_status_changed', array( $this, 'order_status_changed' ), 10, 3 );

			// update the shop info in Jilt once per day
			if ( ! wp_next_scheduled( 'wc_jilt_shop_update' ) ) {
				wp_schedule_event( time(), 'daily' , 'wc_jilt_shop_update' );
			}
			add_action( 'wc_jilt_shop_update', array( $this, 'update_shop') );
		}

		// Save admin options
		if ( is_admin() ) {
			add_action( 'woocommerce_update_options_integration_jilt', array( $this, 'process_admin_options' ) );

			// report connection errors
			add_action( 'admin_notices', array( $this, 'show_connection_notices' ) );
		}
	}


	/**
	 * Sanitize Jilt settings. Removes faux settings, such as `links`.
	 *
	 * @since 1.0.0
	 * @param array $sanitized_fields
	 * @return array
	 */
	public function sanitize_fields( $sanitized_fields ) {

		if ( isset( $sanitized_fields['links'] ) ) {
			unset( $sanitized_fields['links'] );
		}

		return $sanitized_fields;
	}


	/**
	 * Initializes form fields in the format required by WC_Integration
	 *
	 * @since 1.0.0
	 */
	public function init_form_fields() {

		$this->form_fields = array(

			'secret_key' => array(
				'title'   => __( 'Secret Key', 'jilt-for-woocommerce' ),
				'type'    => 'password',
				/* translators: Placeholders: %1$s - <a> tag, %2$s - </a> tag */
				'description' => sprintf( __( 'Get this from your %1$sJilt account%2$s', 'jilt-for-woocommerce' ), '<a href="https://' . wc_jilt()->get_app_hostname() . '/account">', '</a>' ),
			),

			'debug_mode' => array(
				'title'   => __( 'Debug Mode', 'jilt-for-woocommerce' ),
				'type'    => 'select',
				/* translators: Placeholders: %1$s - <a> tag, %2$s - </a> tag */
				'description'    => sprintf( __( 'Save detailed error messages and API requests/responses to the %1$sdebug log%2$s', 'jilt-for-woocommerce' ), '<a href="' . esc_url( SV_WC_Helper::get_wc_log_file_url( wc_jilt()->get_id() ) ) . '">', '</a>' ),
				'default' => self::DEBUG_MODE_OFF,
				'options' => array(
					self::DEBUG_MODE_OFF => _x( 'Off', 'Debug mode off', 'jilt-for-woocommerce' ),
					self::DEBUG_MODE_LOG => __( 'Save to Log', 'jilt-for-woocommerce' ),
				),
			),

			'links' => array(
				'title'       => '',
				'type'        => 'title',
				'description' => $this->get_links_form_field_description(),
			),
		);
	}


	/**
	 * Returns an HTML fragment containing the Jilt external campaigns/dashboard
	 * links for the plugin settings page
	 *
	 * @since 1.0.2-1
	 * @return string HTML fragment
	 */
	private function get_links_form_field_description() {

		/* translators: Placeholders: %1$s - <a> tag, %2$s - </a> tag*/
		$links = sprintf( __( '%1$sGet Support!%2$s', 'jilt-for-woocommerce' ),
			'<a target="_blank" href="' . esc_url( wc_jilt()->get_support_url() ) . '">', '</a>'
		);

		if ( $this->is_linked() ) {
			/* translators: Placeholders: %1$s - <a> tag, %2$s - </a> tag, %3$s - <a> tag, %4$s - </a> tag */
			$links = sprintf( __( '%1$sConfigure my Campaigns%2$s | %3$sView Statistics%4$s', 'jilt-for-woocommerce' ),
				'<a target="_blank" href="' . esc_url( $this->get_jilt_app_url( 'campaigns' ) ) . '">', '</a>',
				'<a target="_blank" href="' . esc_url( $this->get_jilt_app_url( 'dashboard' ) ) . '">', '</a>'
			) . ' | ' . $links;
		}

		return $links;
	}


	/**
	 * Update Jilt public key and shop ID when updating secret key
	 *
	 * @since 1.0.0
	 * @see WC_Settings_API::process_admin_options
	 * @return bool
	 */
	public function process_admin_options() {

		$old    = $this->settings;
		$result = parent::process_admin_options();
		$new    = $this->settings;

		if ( $new['secret_key'] && ( $new['secret_key'] != $old['secret_key'] || ! wc_jilt()->get_integration()->has_connected() || ! wc_jilt()->get_integration()->is_linked() ) ) {
			$this->connect_to_jilt( $new['secret_key'] );
		}

		// disconnecting from Jilt :'(
		if ( ! $new['secret_key'] && $old['secret_key'] ) {
			update_option( 'wc_jilt_public_key', '' );
			update_option( 'wc_jilt_shop_id',    '' );

			wc_jilt()->get_admin_notice_handler()->add_admin_notice( __( 'Shop is now unlinked from Jilt', 'jilt-for-woocommerce' ), 'unlink-notice' );
		}

		// the links that we added to the settings block could be incorrect now
		if ( $new['secret_key'] != $old['secret_key'] ) {
			add_action( 'admin_footer',  array( $this, 'update_jilt_links_js' ) );
		}

		return $result;
	}


	/**
	 * We already show connection error notices when the plugin settings save
	 * post is happening; this method makes those notices more persistent by
	 * showing a connection notice on a regular page load if there's an issue
	 * with the Jilt connection.
	 *
	 * @since 1.0.2-1
	 */
	public function show_connection_notices() {

		// if we're on the Jilt settings page and we're not currently saving the settings (e.g. regular page load), and the plugin is configured
		if ( ! wc_jilt()->is_plugin_settings() || isset( $_POST['save'] ) || ! $this->is_configured() ) {
			return;
		}

		$message = null;

		// call to action based on error state
		if ( ! wc_jilt()->get_integration()->has_connected() ) {

			/* translators: Placeholders: %1$s - <a> tag, %2$s - </a> tag, %3$s - <a> tag, %4$s - </a> tag */
			$message = sprintf( __( 'Please try re-entering your %1$sJilt API Secret Key%2$s or %3$sget in touch with Jilt Support%4$s to help resolve this issue.', 'jilt-for-woocommerce' ),
				'<a target="_blank" href="https://' . wc_jilt()->get_app_hostname() . '/account">',
				'</a>',
				'<a target="_blank" href="' . esc_url( wc_jilt()->get_support_url() ) . '">',
				'</a>'
			);

		} elseif ( ! wc_jilt()->get_integration()->is_linked() ) {

			/* translators: Placeholders: %1$s - <a> tag, %2$s - </a> tag */
			$message = sprintf( __( 'Please %1$sget in touch with Jilt Support%2$s to help resolve this issue.', 'jilt-for-woocommerce' ),
				'<a target="_blank" href="' . esc_url( wc_jilt()->get_support_url( array( 'message' => "I'm having an issue linking my shop to Jilt" ) ) ) . '">',
				'</a>'
			);

		}

		if ( ! empty( $message ) ) {
			wc_jilt()->get_admin_notice_handler()->add_admin_notice(
				/* translators: Placeholders: %1$s - <strong> tag, %2$s - </strong> tag, %3$s - action message */
				sprintf( __( '%1$sError communicating with Jilt%2$s %3$s', 'jilt-for-woocommerce' ), '<strong>', '</strong>', $message ),
				'api-error',
				array( 'notice_class' => 'error' )
			);
		}
	}


	/**
	 * Render javascript to update the Jilt campaign/statistics external links
	 * shown on the Jilt plugin settings page, via JavaScript. This is done
	 * when the shop's connection to Jilt may have changed, since the links are
	 * first written out before any Jilt connection/disconnection is handled.
	 *
	 * @since 1.0.2-1
	 */
	public function update_jilt_links_js() {
		?>
		<script>
			jQuery('#woocommerce_jilt_links + p').html('<?php echo $this->get_links_form_field_description(); ?>');
		</script>
		<?php
	}


	/**
	 * If a $secret_key is provided, attempt to connect to the Jilt API to
	 * retrieve the corresponding Public Key, and link the shop to Jilt
	 *
	 * @since 1.0.0
	 * @param string $secret_key the secret key to use, or empty string
	 * @return true if this shop is successfully connected to Jilt, false otherwise
	 */
	private function connect_to_jilt( $secret_key ) {

		try {

			// remove the previous public key and linked shop id, if any, when the secret key is changed
			update_option( 'wc_jilt_public_key', '' );
			update_option( 'wc_jilt_shop_id',    '' );

			if ( $secret_key ) {
				$this->refresh_public_key();

				if ( is_int( $this->link_shop() ) ) {
					// dismiss the "welcome" message now that we've successfully linked
					wc_jilt()->get_admin_notice_handler()->dismiss_notice( 'get-started-notice' );
					wc_jilt()->get_admin_notice_handler()->add_admin_notice( __( 'Shop is now linked to Jilt!', 'jilt-for-woocommerce' ) );
					return true;
				}
			}

			return false;

		} catch ( SV_WC_API_Exception $exception ) {

			// call to action based on error message
			if ( SV_WC_Helper::str_exists( $exception->getMessage(), 'Invalid API Key provided' ) ) {

				/* translators: Placeholders: %1$s - <a> tag, %2$s - </a> tag, %3$s - <a> tag, %4$s - </a> tag */
				$message = sprintf( __( 'Please try re-entering your %1$sJilt API Secret Key%2$s or %3$sget in touch with Jilt Support%4$s to resolve this issue.', 'jilt-for-woocommerce' ),
					'<a target="_blank" href="https://' . wc_jilt()->get_app_hostname() . '/account">',
					'</a>',
					'<a target="_blank" href="' . esc_url( wc_jilt()->get_support_url( array( 'message' => $exception->getMessage() ) ) ) . '">',
					'</a>' );

			} else {

				/* translators: Placeholders: %1$s - <a> tag, %2$s - </a> tag */
				$message = sprintf( __( 'Please %1$sget in touch with Jilt Support%2$s to resolve this issue', 'jilt-for-woocommerce' ), '<a target="_blank" href="' . esc_url( wc_jilt()->get_support_url( array( 'message' => $exception->getMessage() ) ) ) . '">', '</a>' );
			}

			wc_jilt()->get_admin_notice_handler()->add_admin_notice(
				/* translators: Placeholders: %1$s - <strong> tag, %2$s - </strong> tag, %3$s - error message, %4$s - action message */
				sprintf( __( '%1$sError communicating with Jilt%2$s %3$s. %4$s', 'jilt-for-woocommerce' ), '<strong>', '</strong>', $exception->getMessage(), $message ),
				'api-error',
				array( 'notice_class' => 'error' )
			);

			wc_jilt()->log( "Error communicating with Jilt: {$exception->getMessage()}" );

			return false;
		}
	}


	/** Getter methods ******************************************************/


	/**
	 * Returns the Jilt API instance
	 *
	 * @since 1.0.0
	 * @return WC_Jilt_API the API instance
	 */
	public function get_api() {

		if ( is_null( $this->api ) ) {
			$this->api = new WC_Jilt_API( $this->get_linked_shop_id(), $this->get_secret_key() );
		}

		return $this->api;
	}


	/**
	 * Gets the configured secret key
	 *
	 * @since 1.0.0
	 * @return string the secret key, if set, null otherwise
	 */
	public function get_secret_key() {
		return $this->get_option( 'secret_key' );
	}


	/**
	 * Is the plugin configured?
	 *
	 * @since 1.0.0
	 * @return boolean true if the plugin is configured, false otherwise
	 */
	public function is_configured() {
		return (bool) $this->get_secret_key();
	}


	/**
	 * Has the plugin connected to the Jilt REST API with the current secret key?
	 *
	 * @since 1.0.0
	 * @return boolean true if the plugin has connected to the Jilt REST API
	 *         with the current secret key, false otherwise
	 */
	public function has_connected() {

		// since the public key is returned by the REST API it serves as a
		//  reasonable proxy for whether we've connected
		// note that we get the option directly
		return (bool) get_option( 'wc_jilt_public_key' );
	}


	/**
	 * Returns true if this shop has linked itself to a Jilt user account over
	 * the REST API
	 *
	 * @since 1.0.0
	 * @return boolean true if this shop is linked
	 */
	public function is_linked() {
		return (bool) $this->get_linked_shop_id();
	}


	/**
	 * Get the linked Jilt Shop identifier for this site, if any
	 *
	 * @since 1.0.0
	 * @return int Jilt shop identifier, or null
	 */
	public function get_linked_shop_id() {
		return get_option( 'wc_jilt_shop_id', null );
	}


	/**
	 * Persists the given linked Shop identifier
	 *
	 * @since 1.0.0
	 * @param int $id the linked Shop identifier
	 * @return int the provided $id
	 */
	public function set_linked_shop_id( $id ) {
		update_option( 'wc_jilt_shop_id', $id );
		return $id;
	}


	/**
	 * Returns the current debug mode
	 *
	 * @since 1.0.0
	 * @return string one of DEBUG_MODE_OFF or DEBUG_MODE_LOG
	 */
	public function get_debug_mode() {

		// when updating settings, make sure we have the new value
		if ( isset( $_POST['woocommerce_jilt_debug_mode'] ) ) {
			return $_POST['woocommerce_jilt_debug_mode'];
		}

		$debug_mode = $this->get_option( 'debug_mode' );

		return $debug_mode ? $debug_mode : self::DEBUG_MODE_OFF;
	}


	/**
	 * Returns true if all debugging is disabled
	 *
	 * @since 1.0.0
	 * @return boolean if all debugging is disabled
	 */
	public function debug_off() {
		return self::DEBUG_MODE_OFF === $this->get_debug_mode();
	}


	/**
	 * Returns true if debug logging is enabled
	 *
	 * @since 1.0.0
	 * @return boolean if debug logging is enabled
	 */
	public function debug_log() {
		return self::DEBUG_MODE_LOG === $this->get_debug_mode();
	}


	/** API methods ******************************************************/


	/**
	 * Link this shop to Jilt. The basic algorithm is to first attempt to
	 * create the shop over the Jilt API. If this request fails with a
	 * "Domain has already been taken" error, we try to find it over the Jilt
	 * API by domain, and update with the latest shop data.
	 *
	 * @since 1.0.0
	 * @return int the Jilt linked shop id, or null on network exception or API error
	 */
	public function link_shop() {

		if ( $this->is_configured() ) {

			$args = $this->get_shop_data();

			// set shop owner/email
			$current_user       = wp_get_current_user();
			$args['shop_owner'] = $current_user->user_firstname . ' ' . $current_user->user_lastname;
			$args['email']      = $current_user->user_email;

			try {

				$shop = $this->get_api()->create_shop( $args );

				return $this->set_linked_shop_id( $shop->id );

			} catch ( SV_WC_API_Exception $exception ) {

				if ( SV_WC_Helper::str_exists( $exception->getMessage(), 'Domain has already been taken' ) ) {

					// log the exception and continue attempting to recover
					wc_jilt()->log( "Error communicating with Jilt: {$exception->getMessage()}" );

				} else {

					// for any error other than "Domain has already been taken", display the error and we're done
					return $this->report_api_error( $exception->getMessage() );
				}
			}

			// if we're down here, it means that our attempt to create the
			// shop failed with "domain has already been taken". Lets try to
			// recover gracefully by finding the shop over the API
			try {

				$shop = $this->get_api()->find_shop( array( 'domain' => $args['domain'] ) );

				// no shop found? it might even exist, but the current API user might not have access to it
				if ( ! $shop ) {
					return $this->report_api_error( 'Unable to link shop', false );
				}

			} catch ( SV_WC_API_Exception $exception ) {

				// some other failure
				return $this->report_api_error( $exception->getMessage() );
			}

			// we successfully found our shop. attempt to update it and save the ID
			try {

				// update the linked shop record with the latest settings
				$this->get_api()->update_shop( $args, $shop->id );

			} catch ( SV_WC_API_Exception $exception ) {

				// otherwise, log the exception
				wc_jilt()->log( "Error communicating with Jilt: {$exception->getMessage()}" );
			}

			return $this->set_linked_shop_id( $shop->id );
		}
	}


	/**
	 * Update the shop info in Jilt once per day, useful for keeping track
	 * of which WP/WC versions are in use
	 *
	 * @since 1.0.0
	 */
	public function update_shop() {

		if ( ! $this->is_linked() ) {
			return;
		}

		try {

			// update the linked shop record with the latest settings
			$this->get_api()->update_shop( $this->get_shop_data() );

		} catch ( SV_WC_API_Exception $exception ) {

			// otherwise, log the exception
			wc_jilt()->log( "Error communicating with Jilt: {$exception->getMessage()}" );
		}
	}


	/**
	 * Get base data for creating/updating a linked shop in Jilt
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function get_shop_data() {

		$theme         = wp_get_theme();
		$base_location = wc_get_base_location();

		return array(
			'domain'              => parse_url( get_home_url(), PHP_URL_HOST ),
			'profile_type'        => 'woocommerce',
			'woocommerce_version' => WC()->version,
			'wordpress_version'   => get_bloginfo( 'version' ),
			'integration_version' => wc_jilt()->get_version(),
			'name'                => get_bloginfo( 'name' ),
			'main_theme'          => $theme->name,
			'currency'            => get_woocommerce_currency(),
			'province_code'       => $base_location['state'],
			'country_code'        => $base_location['country'],
			'timezone'            => wc_timezone_string(),
		);
	}


	/**
	 * Update related Jilt order when order status changes
	 *
	 * @since 1.0.0
	 * @param int $order_id order ID
	 * @param string $old_status
	 * @param string $new_status
	 */
	public function order_status_changed( $order_id, $old_status, $new_status ) {

		$jilt_order_id = get_post_meta( $order_id, '_wc_jilt_order_id', true );

		// bail out if this order is not associated with a Jilt order
		if ( ! $jilt_order_id ) {
			return;
		}

		$order = new WC_Jilt_Order( $order_id );

		$params = array(
			'status'           => $new_status,
			'financial_status' => $order->get_financial_status(),
		);

		// when an order moves from pending/failed status to a paid (processing/completed) or on-hold status, mark it as placed
		// this matches the WC core behavior where product sales are recorded when an order is moved to on-hold/processing/completed statuses
		// see WC_Abstract_Order::update_status()
		if ( in_array( $old_status, array( 'pending', 'failed' ), true ) && ( $order->is_paid() || $order->has_status( 'on-hold' ) ) ) {

			// in UTC
			$placed_at = current_time( 'mysql', true );

			update_post_meta( $order->id, '_wc_jilt_placed_at', $placed_at );

			$params['placed_at'] = strtotime( $placed_at );
		}

		// handle order cancellation
		if ( 'cancelled' == $new_status ) {
			$params['cancelled_at'] = current_time( 'timestamp', true );
		}

		// update Jilt order details
		try {

			$this->get_api()->update_order( $jilt_order_id, $params );

		} catch ( SV_WC_API_Exception $exception ) {

			wc_jilt()->log( "Error communicating with Jilt: {$exception->getMessage()}" );
		}
	}


	/**
	 * Get and persist the public key for the current API user from the Jilt REST
	 * API
	 *
	 * @since 1.0.0
	 * @return string the public key
	 * @throws SV_WC_API_Exception on network exception or API error
	 */
	public function refresh_public_key() {
		return $this->get_public_key( true );
	}


	/**
	 * Gets the configured public key, optionally refreshing from the Jilt REST
	 * API if $refresh is true
	 *
	 * @since 1.0.0
	 * @param boolean $refresh true if the current API user public key should
	 *        be fetched from the Jilt API
	 * @return string the public key, if set
	 * @throws SV_WC_API_Exception on network exception or API error
	 */
	public function get_public_key( $refresh = false ) {

		$public_key = get_option( 'wc_jilt_public_key', null );

		if ( ( $refresh || ! $public_key ) && $this->is_configured() ) {
			$public_key = $this->get_api()->get_public_key();
			update_option( 'wc_jilt_public_key', $public_key );
		}

		return $public_key;
	}


	/** Helper methods ******************************************************/


	/**
	 * Report an API error message in an admin notice with a link to the Jilt
	 * support page. Optionally log error.
	 *
	 * @since 1.0.0
	 * @param string $message the API error message
	 * @param boolean $log optionally log the error, defaults to true
	 */
	private function report_api_error( $message, $log = true ) {

		/* translators: Placeholders: %1$s - <strong> tag, %2$s - </strong> tag, %3$s - an error message, %4$s - <a> tag, %5$s - </a> tag */
		$notice = sprintf(
			'%1$sError linking to Jilt%2$s %3$s. Please %4$sget in touch with Jilt Support%5$s to resolve this issue.',
			'<strong>',
			'</strong>',
			$message,
			'<a target="_blank" href="' . esc_url( wc_jilt()->get_support_url( array( 'message' => $message ) ) ) . '">',
			'</a>'
		);

		wc_jilt()->get_admin_notice_handler()->add_admin_notice(
			$notice,
			'api-error',
			array( 'notice_class' => 'error' )
		);

		if ( $log ) {
			wc_jilt()->log( "Error communicating with Jilt: {$message}" );
		}
	}


	/**
	 * Return the URL to the specified page within the Jilt web app, useful
	 * for direct linking to internal pages, like campaigns
	 *
	 * @since 1.0.0
	 * @param string $page page URL partial, e.g. 'dashboard'
	 * @return string
	 */
	public function get_jilt_app_url( $page = '' ) {

		return sprintf( 'https://' . wc_jilt()->get_app_hostname() . '/shops/%1$d/%2$s', (int) $this->get_linked_shop_id(), rawurlencode( $page ) );
	}


}
